#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import warnings
from builtins import int, open, range, str

from future import standard_library

import numpy as np
import scipy.spatial.distance as distance
from scipy.cluster.hierarchy import linkage
from six import string_types

from .hsic_lasso import hsic_lasso
from .input_data import input_csv_file, input_matlab_file, input_tsv_file
from .nlars import nlars
from .plot_figure import plot_dendrogram, plot_heatmap, plot_path

standard_library.install_aliases()


class HSICLasso(object):
    def __init__(self):
        self.input_file = None
        self.X_in = None
        self.Y_in = None
        self.X = None
        self.Xty = None
        self.path = None
        self.beta = None
        self.A = None
        self.A_neighbors = None
        self.A_neighbors_score = None
        self.lam = None
        self.featname = None
        self.linkage_dist = None
        self.hclust_featname = None
        self.hclust_featnameindex = None
        self.max_neighbors = 10

    def input(self, *args, **_3to2kwargs):
        if 'output_list' in _3to2kwargs: output_list = _3to2kwargs['output_list']; del _3to2kwargs['output_list']
        else: output_list = ['class']
        self._check_args(args)
        if isinstance(args[0], string_types):
            self._input_data_file(args[0], output_list)
        elif isinstance(args[0], list):
            self._input_data_list(args[0], args[1])
        elif isinstance(args[0], np.ndarray):
            self._input_data_ndarray(args[0], args[1])
        else:
            pass
        if self.X_in is None or self.Y_in is None:
            raise ValueError("Check your input data")
        self._check_shape()
        return True

    def regression(self, num_feat=5, B=0, M=1, discrete_x=False, max_neighbors=10, n_jobs=-1):
        self._run_hsic_lasso(num_feat=num_feat,
                             y_kernel="Gauss",
                             B=B, M=M,
                             discrete_x=discrete_x,
                             max_neighbors=max_neighbors,
                             n_jobs=n_jobs)

        return True

    def classification(self, num_feat=5, B=0, M=1, discrete_x=False, max_neighbors=10, n_jobs=-1):
        self._run_hsic_lasso(num_feat=num_feat,
                             y_kernel="Delta",
                             B=B, M=M,
                             discrete_x=discrete_x,
                             max_neighbors=max_neighbors,
                             n_jobs=n_jobs)

        return True

    def _run_hsic_lasso(self, y_kernel, num_feat, B, M, discrete_x, max_neighbors, n_jobs):
        if self.X_in is None or self.Y_in is None:
            raise UnboundLocalError("Input your data")
        self.max_neighbors = max_neighbors
        n = self.X_in.shape[1]
        B = B if B else n
        x_kernel = "Delta" if discrete_x else "Gauss"
        numblocks = n / B
        discarded = n % B

        if discarded:
            msg = "B {} must be an exact divisor of the number of samples {}. Number \
of blocks {} will be approximated to {}.".format(B, n, numblocks, int(numblocks))
            warnings.warn(msg, RuntimeWarning)
            numblocks = int(numblocks)

        # Number of permutations of the block HSIC
        perms = 1 + bool(numblocks - 1) * (M - 1)

        X, Xty = hsic_lasso(self.X_in, self.Y_in, y_kernel, x_kernel,
                            n_jobs=n_jobs, discarded=discarded, B=B, perms=perms)

        # np.concatenate(self.X, axis = 0) * np.sqrt(1/(numblocks * perms))
        self.X = X * np.sqrt(1 / (numblocks * perms))
        self.Xty = Xty * 1 / (numblocks * perms)

        self.path, self.beta, self.A, self.lam, self.A_neighbors, \
            self.A_neighbors_score = nlars(
                self.X, self.Xty, num_feat, self.max_neighbors)

        return True

    # For kernel Hierarchical Clustering
    def linkage(self, method="ward"):
        if self.A is None:
            raise UnboundLocalError("Run regression/classification first")
        # selected feature name
        featname_index = []
        featname_selected = []
        for i in range(len(self.A) - 1):
            for index in self.A_neighbors[i]:
                if index not in featname_index:
                    featname_index.append(index)
                    featname_selected.append(self.featname[index])
        self.hclust_featname = featname_selected
        self.hclust_featnameindex = featname_index
        sim = np.dot(self.X[:, featname_index].transpose(),
                     self.X[:, featname_index])
        dist = 1 - sim
        dist = np.maximum(0, dist - np.diag(np.diag(dist)))
        dist_sym = (dist + dist.transpose()) / 2.0
        self.linkage_dist = linkage(distance.squareform(dist_sym), method)

        return True

    def dump(self):
        maxval = self.beta[self.A[0]][0]
        print("============================================== HSICLasso : Result ==================================================")
        print("| Order | Feature      | Score | Top-5 Related Feature (Relatedness Score)                                          |")
        for i in range(len(self.A)):
            print("| {:<5} | {:<12} | {:.3f} | {:<12} ({:.3f}), {:<12} ({:.3f}),"
                  " {:<12} ({:.3f}), {:<12} ({:.3f}), {:<12} ({:.3f})|".format(i + 1, self.featname[self.A[i]],
                                                                               self.beta[self.A[i]
                                                                                         ][0] / maxval,
                                                                               self.featname[self.A_neighbors[i][1]
                                                                                             ], self.A_neighbors_score[i][1],
                                                                               self.featname[self.A_neighbors[i][2]
                                                                                             ], self.A_neighbors_score[i][2],
                                                                               self.featname[self.A_neighbors[i][3]
                                                                                             ], self.A_neighbors_score[i][3],
                                                                               self.featname[self.A_neighbors[i][4]
                                                                                             ], self.A_neighbors_score[i][4],
                                                                               self.featname[self.A_neighbors[i][5]], self.A_neighbors_score[i][5]))

        #print("===== HSICLasso : Path ======")
        # for i in range(len(self.A)):
        #    print(self.path[self.A[i], 1:])
        # return True

    def plot_heatmap(self):
        if self.linkage_dist is None or self.hclust_featname is None or self.hclust_featnameindex is None:
            raise UnboundLocalError("Input your data")
        plot_heatmap(self.X_in[self.hclust_featnameindex, :],
                     self.linkage_dist, self.hclust_featname)
        return True

    def plot_dendrogram(self):
        if self.linkage_dist is None or self.hclust_featname is None:
            raise UnboundLocalError("Input your data")
        plot_dendrogram(self.linkage_dist, self.hclust_featname)
        return True

    def plot_path(self):
        if self.path is None or self.beta is None or self.A is None:
            raise UnboundLocalError("Input your data")
        plot_path(self.path, self.beta, self.A)
        return True

    def get_features(self):
        index = self.get_index()

        return [self.featname[i] for i in index]

    def get_features_neighbors(self, feat_index=0, num_neighbors=5):
        index = self.get_index_neighbors(
            feat_index=feat_index, num_neighbors=num_neighbors)

        return [self.featname[i] for i in index]

    def get_index(self):
        return self.A

    def get_index_score(self):
        return self.beta[self.A, -1]

    def get_index_neighbors(self, feat_index=0, num_neighbors=5):
        if feat_index > len(self.A) - 1:
            raise IndexError("Index does not exist")

        num_neighbors = min(num_neighbors, self.max_neighbors)

        return self.A_neighbors[feat_index][1:(num_neighbors + 1)]

    def get_index_neighbors_score(self, feat_index=0, num_neighbors=5):
        if feat_index > len(self.A) - 1:
            raise IndexError("Index does not exist")

        num_neighbors = min(num_neighbors, self.max_neighbors)

        return self.A_neighbors_score[feat_index][1:(num_neighbors + 1)]

    def save_HSICmatrix(self, filename='HSICmatrix.csv'):
        if self.X_in is None or self.Y_in is None:
            raise UnboundLocalError("Input your data")

        self.X, self.X_ty = hsic_lasso(self.X_in, self.Y_in, "Gauss")

        K = np.dot(self.X.transpose(), self.X)

        np.savetxt(filename, K, delimiter=',', fmt='%.7f')

        return True

    def save_score(self, filename='aggregated_score.csv'):
        maxval = self.beta[self.A[0]][0]

        #print(maxval + ' ' + maxval_)
        fout = open(filename, 'w')
        featscore = {}
        featcorrcoeff = {}
        for i in range(len(self.A)):
            HSIC_XY = (self.beta[self.A[i]][0] / maxval)

            if self.featname[self.A[i]] not in featscore:
                featscore[self.featname[self.A[i]]] = HSIC_XY

                corrcoeff = np.corrcoef(self.X_in[self.A[i]], self.Y_in)[0][1]

                featcorrcoeff[self.featname[self.A[i]]] = corrcoeff

            else:
                featscore[self.featname[self.A[i]]] += HSIC_XY

            for j in range(1, self.max_neighbors + 1):
                HSIC_XX = self.A_neighbors_score[i][j]
                if self.featname[self.A_neighbors[i][j]] not in featscore:
                    featscore[self.featname[self.A_neighbors[i][j]]
                              ] = HSIC_XY * HSIC_XX

                    corrcoeff = np.corrcoef(
                        self.X_in[self.A_neighbors[i][j]], self.Y_in)[0][1]

                    featcorrcoeff[self.featname[self.A_neighbors[i]
                                                [j]]] = corrcoeff
                else:
                    featscore[self.featname[self.A_neighbors[i][j]]
                              ] += HSIC_XY * HSIC_XX

        # Sorting decending order
        featscore_sorted = sorted(
            featscore.items(), key=lambda x: x[1], reverse=True)

        # Add Pearson correlation for comparison
        fout.write('Feature,Score,Pearson Corr\n')
        for (key, val) in featscore_sorted:
            fout.write(key + ',' + str(val) + ',' +
                       str(featcorrcoeff[key]) + '\n')

        fout.close()

    def save_param(self, filename='param.csv'):
        # Save parameters
        maxval = self.beta[self.A[0]][0]

        fout = open(filename, 'w')
        sstr = 'Feature,Score,'
        for j in range(1, self.max_neighbors + 1):
            sstr = sstr + 'Neighbor %d, Neighbor %d score,' % (j, j)

        sstr = sstr + '\n'
        fout.write(sstr)
        for i in range(len(self.A)):
            tmp = []
            tmp.append(self.featname[self.A[i]])
            tmp.append(str(self.beta[self.A[i]][0] / maxval))
            for j in range(1, self.max_neighbors + 1):
                tmp.append(str(self.featname[self.A_neighbors[i][j]]))
                tmp.append(str(self.A_neighbors_score[i][j]))

            sstr = ','.join(tmp) + '\n'
            fout.write(sstr)

        fout.close()

    # ========================================

    def _check_args(self, args):
        if len(args) == 0 or len(args) >= 3:
            raise SyntaxError("Input as input_data(file_name) or \
                input_data(X_in, Y_in)")
        elif len(args) == 1:
            if isinstance(args[0], string_types):
                if len(args[0]) <= 4:
                    raise ValueError("Check your file name")
                else:
                    ext = args[0][-4:]
                    if ext == ".csv" or ext == ".tsv" or ext == ".mat":
                        pass
                    else:
                        raise TypeError("Input file is only .csv, .tsv .mat")
            else:
                raise TypeError("File name is only str")
        elif len(args) == 2:
            if isinstance(args[0], string_types):
                raise TypeError("Check arg type")
            elif isinstance(args[0], list):
                if isinstance(args[1], list):
                    pass
                else:
                    raise TypeError("Check arg type")
            elif isinstance(args[0], np.ndarray):
                if isinstance(args[1], np.ndarray):
                    pass
                else:
                    raise TypeError("Check arg type")
            else:
                raise TypeError("Check arg type")
        return True

    def _input_data_file(self, file_name, output_list):
        ext = file_name[-4:]
        if ext == ".csv":
            self.X_in, self.Y_in, self.featname = input_csv_file(
                file_name, output_list=output_list)
        elif ext == ".tsv":
            self.X_in, self.Y_in, self.featname = input_tsv_file(
                file_name, output_list=output_list)
        elif ext == ".mat":
            self.X_in, self.Y_in, self.featname = input_matlab_file(file_name)
        return True

    def _input_data_list(self, X_in, Y_in):
        if isinstance(Y_in[0], list):
            raise ValueError("Check your input data")
        self.X_in = np.array(X_in).T
        self.Y_in = np.array(Y_in).reshape(1, len(Y_in))
        return True

    def _input_data_ndarray(self, X_in, Y_in):
        if len(Y_in.shape) == 2:
            raise ValueError("Check your input data")
        self.X_in = X_in.T
        self.Y_in = Y_in.reshape(1, len(Y_in))
        return True

    def _check_shape(self):
        _, x_col_len = self.X_in.shape
        y_row_len, y_col_len = self.Y_in.shape
        # if y_row_len != 1:
        #    raise ValueError("Check your input data")
        if x_col_len != y_col_len:
            raise ValueError(
                "The number of samples in input and output should be same")
        return True

    def _permute_data(self, seed=None):
        np.random.seed(seed)
        n = self.X_in.shape[1]

        perm = np.random.permutation(n)
        self.X_in = self.X_in[:, perm]
        self.Y_in = self.Y_in[:, perm]



#!/usr/bin/env python
# coding: utf-8

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from future import standard_library
from .api import HSICLasso
from .hsic_lasso import hsic_lasso
from .input_data import input_csv_file, input_tsv_file, input_matlab_file
standard_library.install_aliases()



#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from future import standard_library

import numpy as np

standard_library.install_aliases()


def kernel_delta_norm(X_in_1, X_in_2):
    n_1 = X_in_1.shape[1]
    n_2 = X_in_2.shape[1]
    K = np.zeros((n_1, n_2))

    u_list = np.unique(X_in_1)
    for ind in u_list:
        ind_1 = (X_in_1 == ind)
        ind_2 = (X_in_2 == ind)
        idx = (ind_1 & ind_2.T)
        c_1 = np.sqrt(np.count_nonzero(ind_1))
        c_2 = np.sqrt(np.count_nonzero(ind_2))
        K[idx] = 1 / c_1 / c_2
    return K


def kernel_delta(X_in_1, X_in_2):
    n_1 = X_in_1.shape[1]
    n_2 = X_in_2.shape[1]
    K = np.zeros((n_1, n_2))
    u_list = np.unique(X_in_1)
    for ind in u_list:
        ind_1 = (X_in_1 == ind)
        ind_2 = (X_in_2 == ind)
        idx = (ind_1 & ind_2.T)
        K[idx] = 1
    return K


def kernel_gaussian(X_in_1, X_in_2, sigma):
    X_in_12 = np.sum(np.power(X_in_1, 2), 0)
    X_in_12 = np.expand_dims(X_in_12, 0)
    X_in_22 = np.sum(np.power(X_in_2, 2), 0)
    X_in_22 = np.expand_dims(X_in_22, 0)
    dist_2 = (X_in_12 + X_in_22.T) - 2 * np.dot(X_in_1.T, X_in_2)
    K = np.exp(-dist_2 / (2 * np.power(sigma, 2)))
    return K



#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from builtins import dict, range

from future import standard_library

import numpy as np
from joblib import Parallel, delayed

from .kernel_tools import kernel_delta_norm, kernel_gaussian

standard_library.install_aliases()


def compute_input_matrix(X_in, feature_idx, B, n, discarded, perms, x_kernel):

    H = np.eye(B) - 1 / B * np.ones(B)
    X = np.zeros((n * B * perms, 1))

    st = 0
    ed = B ** 2
    index = np.arange(n)
    for p in range(perms):
        np.random.seed(p)
        index = np.random.permutation(index)

        for i in range(0, n - discarded, B):
            j = min(n, i + B)

            # Normalization
            if x_kernel == 'Gauss':
                XX = X_in[index[i:j]] / \
                    (X_in[index[i:j]].std() + 10e-20) * \
                    np.sqrt(float(B - 1) / B)
            else:
                XX = X_in[index[i:j]]

            XX = XX.reshape((1, B))

            if x_kernel == 'Gauss':
                Kx = kernel_gaussian(XX[0, None], XX[0, None], 1.0)
            elif x_kernel == 'Delta':
                Kx = kernel_delta_norm(XX[0, None], XX[0, None])

            tmp = np.dot(np.dot(H, Kx), H)

            # Normalize HSIC tr(tmp*tmp) = 1
            tmp = tmp / np.linalg.norm(tmp, 'fro')
            X[st:ed, 0] = tmp.flatten()
            st += B ** 2
            ed += B ** 2

    return (feature_idx, X.flatten())


def hsic_lasso(X_in, Y_in, y_kernel, x_kernel='Gauss', n_jobs=-1, discarded=0, B=0, perms=1):
    """
    Input:
        X_in      input_data
        Y_in      target_data
        y_kernel  We employ the Gaussian kernel for inputs. For output kernels,
                  we use the Gaussian kernel for regression cases and
                  the delta kernel for classification problems.
    Output:
        X         matrix of size d x (n * B (or n) * perms)
        X_ty      vector of size d x 1
    """
    d, n = X_in.shape
    dy = Y_in.shape[0]

    # Centering matrix
    H = np.eye(B) - 1 / B * np.ones(B)
    lf = np.zeros((n * B * perms, 1))
    index = np.arange(n)
    st = 0
    ed = B**2
    for p in range(perms):
        np.random.seed(p)
        index = np.random.permutation(index)

        for i in range(0, n - discarded, B):
            j = min(n, i + B)

            if y_kernel == "Delta":
                if dy > 1:
                    raise RuntimeError(
                        "Delta kernel only supports 1 dimensional class labels.")

                L = kernel_delta_norm(Y_in[:, index[i:j]], Y_in[:, index[i:j]])
            elif y_kernel == "Gauss":
                YY = Y_in[:, index[i:j]] / (Y_in[:, index[i:j]].std(1)
                                            [:, None] + 10e-20) * np.sqrt(float(B - 1) / B)
                L = kernel_gaussian(YY, YY, np.sqrt(dy))

            L = np.dot(H, np.dot(L, H))

            # Normalize HSIC tr(L*L) = 1
            L = L / np.linalg.norm(L, 'fro')

            lf[st:ed, 0] = L.flatten()
            st += B**2
            ed += B**2

    # Preparing design matrix for HSIC Lars
    result = Parallel(n_jobs=n_jobs)([delayed(compute_input_matrix)(
        X_in[k, :], k, B, n, discarded, perms, x_kernel) for k in range(d)])
    result = dict(result)

    X = np.array([result[k] for k in range(d)]).T
    X_ty = np.dot(X.T, lf)

    return X, X_ty



#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from builtins import range

from future import standard_library

import numpy as np
from scipy.sparse import lil_matrix

standard_library.install_aliases()


def nlars(X, X_ty, num_feat, max_neighbors):
    """
    We used the Nonnegative LARS solver.
    http://www2.imm.dtu.dk/pubdb/views/edoc_download.php/5523/zip/imm5523.zip

    Solves the problem argmin_beta 1/2||y-X*beta||_2^2  s.t. beta>=0.
    The problem is solved using a modification of the Least Angle Regression
    and Selection algorithm.
    As such the entire regularization path for the LASSO problem
    min 1/2||y-X*beta||_2^2 + lambda|beta|_1  s.t. beta>=0
    for all values of lambda is given in path.

    Input:
        X            matrix of size D x D
        X_ty         vector of size D x 1
        num_feat     the number of features you want to extract
    Output:
        path         the entire solution path
        beta         D x 1 solution vector
        A            selected features
        A_neighbors  related features of the selected features in A
        lam(lambda)  regularization value at beginning of step corresponds
                     to value of negative gradient
    """
    n, d = X.shape

    A = []
    A_neighbors = []
    A_neighbors_score = []
    beta = np.zeros((d, 1))
    path = lil_matrix((d, 4 * d))
    lam = np.zeros((1, 4 * d))

    I = list(range(d))

    XtXbeta = np.dot(X.transpose(), np.dot(X, beta))
    c = X_ty - XtXbeta
    j = c.argmax()
    C = c[j]
    A.append(I[j])
    I.remove(I[j])

    if len(C) == 0:
        lam[0] = 0
    else:
        lam[0, 0] = C[0]

    k = 0
    while sum(c[A]) / len(A) >= 1e-9 and len(A) < num_feat + 1:
        s = np.ones((len(A), 1))
        w = np.linalg.solve(np.dot(X[:, A].transpose(), X[:, A]), s)
        XtXw = np.dot(X.transpose(), np.dot(X[:, A], w))

        gamma1 = (C - c[I]) / (XtXw[A[0]] - XtXw[I])
        gamma2 = -beta[A] / (w)
        gamma3 = np.zeros((1, 1))
        gamma3[0] = c[A[0]] / (XtXw[A[0]])
        gamma = np.concatenate((np.concatenate((gamma1, gamma2)), gamma3))

        gamma[gamma <= 1e-9] = np.inf
        t = gamma.argmin()
        mu = min(gamma)

        beta[A] = beta[A] + mu * w

        if t >= len(gamma1) and t < (len(gamma1) + len(gamma2)):
            lasso_cond = 1
            j = t - len(gamma1)
            I.append(A[j])
            A.remove(A[j])
        else:
            lasso_cond = 0

        XtXbeta = np.dot(X.transpose(), np.dot(X, beta))
        c = X_ty - XtXbeta
        j = np.argmax(c[I])
        C = max(c[I])

        k += 1
        path[:, k] = beta

        if len(C) == 0:
            lam[k] = 0
        else:
            lam[0, k] = C[0]
        if lasso_cond == 0:
            A.append(I[j])
            I.remove(I[j])

    # We run numfeat + 1 iteration to update beta and path information
    # Then, we return only numfeat features
    if len(A) > num_feat:
        A.pop()

    # Sort A with respect to beta
    s = beta[A]
    sort_index = sorted(range(len(s)), key=lambda k: s[k], reverse=True)

    A_sorted = [A[i] for i in sort_index]

    # Find nighbors of selected features
    XtXA = np.dot(X.transpose(), X[:, A_sorted])

    # Search up to 10 nighbors
    num_neighbors = max_neighbors + 1
    for i in range(0, len(A_sorted)):
        tmp = XtXA[:, i]
        sort_index = sorted(
            range(len(tmp)), key=lambda k: tmp[k], reverse=True)
        A_neighbors.append(sort_index[0:num_neighbors])
        A_neighbors_score.append(tmp[sort_index[0:num_neighbors]])

    path_final = path[:, 0:(k + 1)].toarray()
    lam_final = lam[0:(k + 1)]

    return path_final, beta, A_sorted, lam_final, A_neighbors, A_neighbors_score



#!/usr/bin.env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import unittest

import numpy as np
from future import standard_library

from pyHSICLasso import HSICLasso

standard_library.install_aliases()



class InputTest(unittest.TestCase):
    def setUp(self):
        self.hsic_lasso = HSICLasso()

    def test_check_arg(self):
        with self.assertRaises(SyntaxError):
            self.hsic_lasso._check_args([])
        with self.assertRaises(SyntaxError):
            self.hsic_lasso._check_args([1, 2, 3])
        with self.assertRaises(ValueError):
            self.hsic_lasso._check_args(["txt"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args(["hoge.txt"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args(["hogecsv"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([123])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([[1, 2, 3]])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([np.array([1, 2, 3])])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args(["hoge", "hoge"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args(["hoge", [1, 2, 3]])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([[1, 2, 3], "hoge"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args(["hoge", np.array([1, 2, 3])])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([np.array([1, 2, 3]), "hoge"])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([123, [1, 2, 3]])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([[1, 2, 3], 123])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([123, np.array([1, 2, 3])])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([np.array([1, 2, 3]), 123])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([[1, 2, 3], np.array([1, 2, 3])])
        with self.assertRaises(TypeError):
            self.hsic_lasso._check_args([np.array([1, 2, 3]), [1, 2, 3]])

        self.assertTrue(self.hsic_lasso._check_args(["hoge.csv"]))
        self.assertTrue(self.hsic_lasso._check_args(["hoge.tsv"]))
        self.assertTrue(self.hsic_lasso._check_args(["hoge.mat"]))
        self.assertTrue(self.hsic_lasso._check_args([np.array([1, 2, 3]),
                                                    np.array([1, 2, 3])]))
        self.assertTrue(self.hsic_lasso._check_args([[1, 2, 3], [1, 2, 3]]))

    def test_input_data_file(self):
        self.assertTrue("./tests/test_data/csv_data.csv")
        self.assertTrue("./tests/test_data/tsv_data.tsv")
        self.assertTrue("./tests/test_data/mat_data.mat")

    def test_input_data_list(self):
        self.hsic_lasso._input_data_list([[1, 1, 1],
                                          [2, 2, 2]
                                          ],
                                         [1, 2])
        X_in_row, X_in_col = self.hsic_lasso.X_in.shape
        Y_in_row, Y_in_col = self.hsic_lasso.Y_in.shape
        self.assertEqual(X_in_row, 3)
        self.assertEqual(X_in_col, 2)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 2)

        self.hsic_lasso._input_data_list([[1, 1, 1, 1, 1],
                                          [2, 2, 2, 2, 2],
                                          [3, 3, 3, 3, 3],
                                          [4, 4, 4, 4, 4]
                                          ],
                                         [1, 2, 3, 4])
        X_in_row, X_in_col = self.hsic_lasso.X_in.shape
        Y_in_row, Y_in_col = self.hsic_lasso.Y_in.shape
        self.assertEqual(X_in_row, 5)
        self.assertEqual(X_in_col, 4)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 4)

        with self.assertRaises(ValueError):
            self.hsic_lasso._input_data_list([[1, 1, 1, 1, 1],
                                              [2, 2, 2, 2, 2],
                                              [3, 3, 3, 3, 3],
                                              [4, 4, 4, 4, 4]
                                              ],
                                             [[1, 2, 3, 4],
                                              [1, 2, 3, 4]
                                              ])

    def test_input_data_ndarray(self):
        self.hsic_lasso._input_data_ndarray(np.array([[1, 1, 1],
                                                      [2, 2, 2]
                                                      ]),
                                            np.array([1, 2]))
        X_in_row, X_in_col = self.hsic_lasso.X_in.shape
        Y_in_row, Y_in_col = self.hsic_lasso.Y_in.shape
        self.assertEqual(X_in_row, 3)
        self.assertEqual(X_in_col, 2)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 2)

        self.hsic_lasso._input_data_ndarray(np.array([[1, 1, 1, 1, 1],
                                                      [2, 2, 2, 2, 2],
                                                      [3, 3, 3, 3, 3],
                                                      [4, 4, 4, 4, 4]
                                                      ]),
                                            np.array([1, 2, 3, 4]))
        X_in_row, X_in_col = self.hsic_lasso.X_in.shape
        Y_in_row, Y_in_col = self.hsic_lasso.Y_in.shape
        self.assertEqual(X_in_row, 5)
        self.assertEqual(X_in_col, 4)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 4)

        with self.assertRaises(ValueError):
            self.hsic_lasso._input_data_list(np.array([[1, 1, 1, 1, 1],
                                                       [2, 2, 2, 2, 2],
                                                       [3, 3, 3, 3, 3],
                                                       [4, 4, 4, 4, 4]
                                                       ]),
                                             np.array([[1, 2, 3, 4],
                                                       [1, 2, 3, 4]
                                                       ]))

    def test_check_shape(self):
        self.hsic_lasso._input_data_list([[1, 1, 1],
                                          [2, 2, 2]
                                          ],
                                         [1, 2])
        self.assertTrue(self.hsic_lasso._check_shape())

        self.hsic_lasso._input_data_list([[1, 1, 1, 1, 1],
                                          [2, 2, 2, 2, 2],
                                          [3, 3, 3, 3, 3],
                                          [4, 4, 4, 4, 4]
                                          ],
                                         [1, 2, 3, 4])
        self.assertTrue(self.hsic_lasso._check_shape())

        self.hsic_lasso._input_data_list([[1, 1, 1, 1],
                                          [2, 2, 2, 2],
                                          [3, 3, 3, 3],
                                          [4, 4, 4, 4]
                                          ],
                                         [1, 2, 3, 4])
        self.assertTrue(self.hsic_lasso._check_shape())

        self.hsic_lasso._input_data_list([[1, 1, 1, 1, 1],
                                          [2, 2, 2, 2, 2],
                                          [3, 3, 3, 3, 3]
                                          ],
                                         [1, 2, 3, 4])
        with self.assertRaises(ValueError):
            self.hsic_lasso._check_shape()

    def test_input(self):
        self.assertTrue(self.hsic_lasso.input("./tests/test_data/csv_data.csv"))
        self.assertTrue(self.hsic_lasso.input("./tests/test_data/tsv_data.tsv"))
        self.assertTrue(self.hsic_lasso.input("./tests/test_data/matlab_data.mat"))


if __name__ == "__main__":
    unittest.main()



#!/usr/bin.env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import unittest

from future import standard_library
import numpy as np
import warnings

from pyHSICLasso import HSICLasso

standard_library.install_aliases()


class ClassificationTest(unittest.TestCase):
    def setUp(self):
        self.hsic_lasso = HSICLasso()

    def test_classification(self):

        np.random.seed(0)

        with self.assertRaises(UnboundLocalError):
            self.hsic_lasso.classification()

        self.hsic_lasso.input("./tests/test_data/csv_data.csv")
        self.hsic_lasso.classification(5, discrete_x = True, n_jobs = 1)
        self.assertEqual(self.hsic_lasso.A, [764, 1422, 512, 248, 1581])

        self.hsic_lasso.input("./tests/test_data/csv_data.csv")
        self.hsic_lasso.classification(10, discrete_x = True, n_jobs = 1)
        self.assertEqual(self.hsic_lasso.A, [764, 1422, 512, 248, 1581, 
                                             1670, 1771, 896, 779, 266])

        # Blocks
        self.hsic_lasso.input("./tests/test_data/csv_data.csv")
        B = int(self.hsic_lasso.X_in.shape[1]/2)
        self.hsic_lasso.classification(5, B, 10, discrete_x = True)
        self.assertEqual(self.hsic_lasso.A, [764, 1422, 512, 248, 266])

        self.hsic_lasso.input("./tests/test_data/csv_data.csv")
        B = int(self.hsic_lasso.X_in.shape[1]/2)
        self.hsic_lasso.classification(10, B, 10, discrete_x = True)
        self.assertEqual(self.hsic_lasso.A, [764, 1422, 512, 248, 1670, 
                                             1581, 266, 896, 1771, 779])

        # use non-divisor as block size
        with warnings.catch_warnings(record=True) as w:
        
            self.hsic_lasso.input("./tests/test_data/csv_data.csv")
            B = int(self.hsic_lasso.X_in.shape[1]/2) - 1
            n = self.hsic_lasso.X_in.shape[1]
            numblocks = n / B
            
            self.hsic_lasso.classification(10, B, 10, discrete_x = True)
            self.assertEqual(self.hsic_lasso.A, [1422, 764, 512, 248, 1670, 
                                                 1581, 896, 266, 1771, 779])
            self.assertEqual(len(w), 1)
            self.assertEqual(w[-1].category, RuntimeWarning)
            self.assertEqual(str(w[-1].message), "B {} must be an exact divisor of the \
number of samples {}. Number of blocks {} will be approximated to {}.".format(B, n, numblocks, int(numblocks)))

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from builtins import range

from future import standard_library

import pandas as pd
import seaborn as sns
from matplotlib import pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
from scipy.cluster.hierarchy import dendrogram

microarray_cmap = LinearSegmentedColormap('microarray', {
    'red': [(0.0, 1.0, 1.0), (0.5, 0.2, 0.2), (1.0, 0.0, 0.0)],
    'green': [(0.0, 0.0, 0.0), (0.5, 0.2, 0.2), (1.0, 1.0, 1.0)],
    'blue': [(0.0, 0.0, 0.0), (0.5, 0.2, 0.2), (1.0, 0.0, 0.0)],
})

standard_library.install_aliases()


def plot_heatmap(X, row_linkage, featname):
    df = pd.DataFrame(X)
    df.index = featname
    cg = sns.clustermap(df, center=0, row_linkage=row_linkage,
                        method='ward', cmap=microarray_cmap)
    cg.ax_heatmap.set_xticklabels("")
    plt.setp(cg.ax_heatmap.yaxis.get_majorticklabels(), rotation=0)
    plt.setp(cg.ax_heatmap.xaxis.get_majorticklabels(), rotation=90)
    plt.title('Heatmap')
    plt.show()


def plot_dendrogram(linkage, featname):
    dendrogram(linkage, labels=featname)
    plt.title("Dendrogram")
    plt.show()


def plot_path(path, beta, A):
    t = path.sum(0)
    plt.title("HSICLasso Result")
    plt.xlabel("lambda")
    plt.ylabel("cofficients")
    for ind in range(len(A)):
        plt.plot(t, path[A[ind], :], label="{}".format(A[ind] + 1))
    plt.legend()
    plt.show()



#!/usr/bin/env python
# coding: utf-8
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import os

from setuptools import setup, find_packages


here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "README.rst")) as f:
    long_description = f.read()


def _load_requires_from_file(filepath):
    return [pkg_name.rstrip('\r\n') for pkg_name in open(filepath).readlines()]


def _install_requires():
    return _load_requires_from_file('requirements.txt')


def _packages():
    return find_packages(
        exclude=[
            '*.tests',
            '*.tests.*',
            'tests.*',
            'tests'
        ]
    )


setup(
    name="pyHSICLasso",
    version="1.3.1",
    author="Makoto Yamada",
    author_email="makoto.yamada@riken.jp",
    url="http://www.makotoyamada-ml.com/hsiclasso.html",
    description="supervised feature selection considering the dependency of\
        nonlinear input and output.",
    long_description=long_description,
    download_url="https://github.com/suecharo/pyHSICLasso",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Scientific/Engineering",
    ],
    platforms=["python2.7", "python3.4", "python3.5", "python3.6"],
    license="MIT",
    keywords="HSIC Lasso HSICLasso feature-selection data-science",
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    install_requires=_install_requires(),
    packages=_packages()
)



#!/usr/bin.env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

import unittest

from future import standard_library

from pyHSICLasso import input_csv_file, input_matlab_file, input_tsv_file

standard_library.install_aliases()



class InputDataTest(unittest.TestCase):
    def test_input_csv(self):
        X_in, Y_in, featname = input_csv_file("./tests/test_data/csv_data.csv")
        X_in_row, X_in_col = X_in.shape
        Y_in_row, Y_in_col = Y_in.shape
        self.assertEqual(X_in_row, 2000)
        self.assertEqual(X_in_col, 62)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 62)

    def test_input_tsv(self):
        X_in, Y_in, featname = input_tsv_file("./tests/test_data/tsv_data.tsv")
        X_in_row, X_in_col = X_in.shape
        Y_in_row, Y_in_col = Y_in.shape
        self.assertEqual(X_in_row, 2000)
        self.assertEqual(X_in_col, 62)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 62)

    def test_input_matlab(self):
        X_in, Y_in, featname = input_matlab_file("./tests/test_data/matlab_data.mat")
        X_in_row, X_in_col = X_in.shape
        Y_in_row, Y_in_col = Y_in.shape
        self.assertEqual(X_in_row, 2000)
        self.assertEqual(X_in_col, 100)
        self.assertEqual(Y_in_row, 1)
        self.assertEqual(Y_in_col, 100)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# coding: utf-8

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)

from builtins import range

from future import standard_library

import pandas as pd
from scipy import io as spio

standard_library.install_aliases()


def input_csv_file(file_name, output_list=['class']):
    return input_txt_file(file_name, output_list, ',')


def input_tsv_file(file_name, output_list=['class']):
    return input_txt_file(file_name, output_list, '\t')


def input_txt_file(file_name, output_list, sep):
    df = pd.read_csv(file_name, sep=sep)

    # Store the column name (Feature name)
    featname = df.columns.tolist()
    input_index = list(range(0, len(featname)))
    output_index = []

    for output_name in output_list:
        if output_name in featname:
            tmp = featname.index(output_name)
            output_index.append(tmp)
            input_index.pop(tmp)
        else:
            raise ValueError("Output variable, %s, not found" % (output_name))

    for output_name in output_list:
        featname.remove(output_name)

    X_in = df.ix[:, input_index].values.T

    if len(output_index) == 1:
        Y_in = df.ix[:, output_index].values.reshape(1, len(df.index))
    else:
        Y_in = df.ix[:, output_index].values.T

    return X_in, Y_in, featname


def input_matlab_file(file_name):
    data = spio.loadmat(file_name)

    if "X" in data.keys() and "Y" in data.keys():
        X_in = data["X"]
        Y_in = data["Y"]
    elif "X_in" in data.keys() and "Y_in" in data.keys():
        X_in = data["X_in"]
        Y_in = data["Y_in"]
    elif "x" in data.keys() and "y" in data.keys():
        X_in = data["x"]
        Y_in = data["y"]
    elif "x_in" in data.keys() and "y_in" in data.keys():
        X_in = data["x_in"]
        Y_in = data["y_in"]
    else:
        raise KeyError("not find input data")

    # Create feature list
    d = X_in.shape[0]
    featname = [('%d' % i) for i in range(1, d + 1)]

    return X_in, Y_in, featname
